/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// author: nazgul (Kamil Skalski) 
//         nazgul@omega.pl
//

// QT headers
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qcheckbox.h>

// Kadu headers
#include <kadu.h>

// private headers
#include "gui.h"
#include "spellchecker.h"
#include "config_dialog.h"

Gui::Gui() {
  // plug into Kadu GUI
  checkerMenu = new QPopupMenu(kadu);
  checkerMenu -> insertItem( icons_manager.loadIcon("History"), 
                             tr("&Languages"), this, SLOT(ChooseLangs()) );
  checkerMenu -> insertItem( icons_manager.loadIcon("Configuration"), 
                             tr("&Settings"), this, SLOT(OpenSettings()) );

  menuId = kadu -> menuBar() -> insertItem( tr("&Spellchecker"), checkerMenu);
  interface = this;
  languagesDialog = NULL;
  settingsDialog = NULL;
}

Gui::~Gui() {
  if ( languagesDialog != NULL )
    delete languagesDialog;
  if ( settingsDialog != NULL )
    delete settingsDialog;
  kadu->menuBar()->removeItem(menuId);
  delete checkerMenu;
}

void Gui::OpenSettings()
{
  if( settingsDialog == NULL)
    {
      settingsDialog = new SettingsDialog();
      connect( settingsDialog, SIGNAL(destroyed()), 
               this, SLOT(SettingsDestroyed()) );
      settingsDialog -> show();
    }
  else
    settingsDialog -> setActiveWindow();
}

void Gui::ChooseLangs()
{
  if( languagesDialog == NULL)
    {
      languagesDialog = new LanguagesDialog();
      connect( languagesDialog, SIGNAL(destroyed()), 
               this, SLOT(LanguagesDestroyed()) );
      languagesDialog -> show();
    }
  else
    languagesDialog -> setActiveWindow();
}

void Gui::LanguagesDestroyed() {
  languagesDialog = NULL;
}

void Gui::SettingsDestroyed() {
  settingsDialog = NULL;
}

LanguagesDialog::LanguagesDialog() : QDialog(NULL,NULL)
{
  setWFlags( Qt::WDestructiveClose );
  resize( 300, 200 );
  setCaption( tr("Manage languages") );
	
  QHBoxLayout* layout = new QHBoxLayout(this);
  QVBoxLayout* availableLayout = new QVBoxLayout(layout);
  QVBoxLayout* checkedLayout = new QVBoxLayout(layout);	
		
  QLabel* availableLabel = new QLabel(this);
  availableLabel -> setText( QString( "<b>" + tr("Available languages") + "</b>" ) );
	
  availableListBox = new QListBox(this);
  availableListBox -> insertStringList( spellcheck -> NotCheckedLanguages() );
  connect( availableListBox, SIGNAL(doubleClicked(QListBoxItem*)),
           this, SLOT(loadItem(QListBoxItem*)));
	
  QButton* addButton = new QPushButton(this);
  addButton -> setText( tr("Add") );
  connect( addButton, SIGNAL(clicked()), this, SLOT(loadSelectedItem()));
	
  availableLayout -> addWidget( availableLabel );
  availableLayout -> addWidget( availableListBox );
  availableLayout -> addWidget( addButton );

  QLabel* checkedLabel = new QLabel(this);
  checkedLabel -> setText( QString("<b>" + tr("Checked languages") + "</b>") );

  checkedListBox = new QListBox(this);
  checkedListBox -> insertStringList( spellcheck -> CheckedLanguages());	
  connect( checkedListBox, SIGNAL(doubleClicked(QListBoxItem*)),
           this, SLOT(unloadItem(QListBoxItem*)) );

  QButton* removeButton = new QPushButton(this);
  removeButton -> setText( tr("Remove") );
  connect( removeButton, SIGNAL(clicked()),
           this, SLOT(unloadSelectedItem()));

  checkedLayout -> addWidget( checkedLabel );
  checkedLayout -> addWidget( checkedListBox );
  checkedLayout -> addWidget( removeButton );
}

void LanguagesDialog::loadItem(QListBoxItem* item)
{
  QString langName = item -> text();
  if ( spellcheck -> AddCheckedLang( langName ) ) {
    checkedListBox -> insertItem( langName );
    availableListBox -> removeItem( availableListBox -> currentItem() );
  }
}

void LanguagesDialog::unloadItem(QListBoxItem* item)
{
  QString langName = item -> text();
  availableListBox -> insertItem( langName );
  checkedListBox -> removeItem( checkedListBox -> currentItem() );
  spellcheck -> RemoveCheckedLang( langName );
}

void LanguagesDialog::loadSelectedItem()
{
  int current = availableListBox -> currentItem();
  if( current >= 0 )
    loadItem( availableListBox -> item( current ) );
}

void LanguagesDialog::unloadSelectedItem()
{
  int current = checkedListBox -> currentItem();
  if( current >= 0 )
    unloadItem( checkedListBox -> item( current ) );
}


SettingsDialog::SettingsDialog() : QDialog(NULL,NULL)
{
  setWFlags( Qt::WDestructiveClose );
  resize( 400, 200 );
  setCaption( tr("Choose spellchecker settings") );

  QVBoxLayout* layout = new QVBoxLayout(this);
  QColor colorMark("#FF0101");
  colorMark = spellcheck -> config -> readColorEntry( "Mark", "Color", 
                                                      &colorMark );
  colorButton = new ColorButton( colorMark, this, "markColor" );
  boldBox = new QCheckBox( tr("Bold"), this, "boldOption" );
  boldBox -> setChecked( spellcheck -> config -> 
                         readBoolEntry( "Mark", "Bold", false ) );

  italicBox = new QCheckBox( tr("Italic"), this, "italicOption" );
  italicBox -> setChecked( spellcheck -> config -> 
                         readBoolEntry( "Mark", "Italic", false ) );

  underlineBox = new QCheckBox( tr("Underline"), this, "undelineOption" );
  underlineBox -> setChecked( spellcheck -> config -> 
                         readBoolEntry( "Mark", "Underline", false ) );

  accentsBox = new QCheckBox( tr("Ignore accents"), this );
  accentsBox -> setChecked( spellcheck -> config -> 
                         readBoolEntry( "Ignore", "Accents", false ) );

  caseBox = new QCheckBox( tr("Ignore case"), this );
  caseBox -> setChecked( spellcheck -> config -> 
                         readBoolEntry( "Ignore", "Case", false ) );

  QPushButton* okButton = new QPushButton( tr("Ok"), this, "ok" );
  connect( okButton, SIGNAL(clicked()), this, SLOT(Accept()) );
  QPushButton* cancelButton = new QPushButton( tr("Cancel"), this, "cancel" );
  connect( cancelButton, SIGNAL(clicked()), this, SLOT(Cancel()) );

  HotKey* addWord = new HotKey( this );

  layout -> addWidget( new QLabel( tr("Options for marking of misspelled words"),
                                   this) );
  layout -> addWidget( colorButton );
  layout -> addWidget( boldBox );
  layout -> addWidget( italicBox );
  layout -> addWidget( underlineBox );
  layout -> addWidget( new QLabel( tr("Behavior of spellchecking"), this ) );
  layout -> addWidget( accentsBox );
  layout -> addWidget( caseBox );
  layout -> addWidget( okButton );
  layout -> addWidget( cancelButton );
  layout -> addWidget( new QLabel( tr("Adding a word"), this ) );
  layout -> addWidget( addWord );
}

void SettingsDialog::Accept() 
{
  spellcheck -> ChangeMarkColor( colorButton -> color() );
  spellcheck -> config -> writeEntry( "Mark", "Bold", 
                                      boldBox -> isChecked() );
  spellcheck -> config -> writeEntry( "Mark", "Italic", 
                                      italicBox -> isChecked() );
  spellcheck -> config -> writeEntry( "Mark", "Underline", 
                                      underlineBox -> isChecked() );
  spellcheck -> config -> writeEntry( "Ignore", "Accents", 
                                      accentsBox -> isChecked() );
  spellcheck -> config -> writeEntry( "Ignore", "Case", 
                                      caseBox -> isChecked() );

  spellcheck -> BuildMarkTag();
  spellcheck -> BuildCheckers();
  close();
}

void SettingsDialog::Cancel()
{
  close();
}

Gui* interface;
