/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 ***************************************************************************/

/*
        author: nazgul (Kamil Skalski) 
                        nazgul@omega.pl

        Pawe (Googie) Salawa
        boogie@scripts.one.pl
                        is a maintainer and developer
                        of this code since 13.06.2004
*/


/*         general purpose macros to make things shorter         */
#define VAR(v,x)                __typeof(x) v=x
#define FOREACH(i,c) for(VAR(i, (c).begin()); i!=(c).end(); ++i)
/*****************************************************************/

// libraries headers
#include <vector>
using namespace std;

#include <stdio.h>
#include <aspell.h>

// QT headers
#include <qtimer.h>
#include <qcolor.h>

// Kadu headers
#include <events.h>
#include <message_box.h>
#include <chat.h>
#include <misc.h>
#include <kadu.h>

// private headers
#include "spellchecker.h"
#include "gui.h"

// private definitions
#define MODULE_SPELLCHECKER_VERSION 0.9

// HTML mark for mispelled words
const char* endMark = "</span>";

extern "C" int spellchecker_init()
{
    spellcheck = new SpellChecker();

    // use configuration settings to create spellcheckers for languages
    if ( !spellcheck->BuildCheckers() ) {
        spellcheck = NULL;
        interface = NULL;
        return 1;
    }
    else {
        interface = new Gui();    

        QObject::connect(chat_manager, 
        SIGNAL(chatCreated(const UinsList&)), 
            spellcheck, 
            SLOT(ChatCreated(const UinsList&)));
        
        return 0;
    }
}

extern "C" void spellchecker_close()
{
    if ( spellcheck ) {
        QObject::disconnect(chat_manager, 
        SIGNAL(chatCreated(const UinsList&)), 
            spellcheck, 
            SLOT(ChatCreated(const UinsList&)));

        delete interface;
        delete spellcheck;
    }
}

SpellChecker::SpellChecker() : QObject()
{
    // this timer will wake up spell checker to analyze chat windows'
    // input and mark spelling mistakes
    myWakeupTimer = new QTimer( this );
    connect( myWakeupTimer, SIGNAL(timeout()), this, SLOT(ExecuteChecking()) );

    // prepare configuration of spellchecker
    spellConfig = new_aspell_config();
    aspell_config_replace( spellConfig, "encoding", "utf-8" ); 

    // load configuration file
    config = new ConfigFile( ggPath( QString("spellchecker.conf") ) );

    // load mark settings
    BuildMarkTag();
}

SpellChecker::~SpellChecker()
{
    myWakeupTimer->stop();
    disconnect( myWakeupTimer, SIGNAL(timeout()), this, SLOT(ExecuteChecking()));

    VAR ( openChats, chat_manager->chats () );
    FOREACH ( chat, openChats ) CleanMessage( *chat );

    delete_aspell_config( spellConfig );
    SaveConfig();
    delete config;
    
    delete myWakeupTimer;
    FOREACH( checker, checkers ) delete_aspell_speller( checker->second );
}

QStringList SpellChecker::NotCheckedLanguages()
{
    QStringList result;
    AspellDictInfoList* dlist;
    AspellDictInfoEnumeration* dels;
    const AspellDictInfo* entry;

    /* the returned pointer should _not_ need to be deleted */
    dlist = get_aspell_dict_info_list( spellConfig );

    dels = aspell_dict_info_list_elements(dlist);
    while ( (entry = aspell_dict_info_enumeration_next( dels )) != 0) 
    {
        if ( checkers.find( entry->name ) == checkers.end() )
            result.push_back( entry->name );
    }
    delete_aspell_dict_info_enumeration(dels);

    return result;
}

QStringList SpellChecker::CheckedLanguages()
{
    QStringList result;
    FOREACH( checker, checkers ) result.push_back( checker->first );
    return result;
}

bool SpellChecker::AddCheckedLang( QString& name )
{
    if ( checkers.find( name ) != checkers.end() ) return true;

    aspell_config_replace( spellConfig, "lang", name ); 

    // create spell checker using prepared configuration
    AspellCanHaveError* possibleErr = new_aspell_speller( spellConfig ); 
    if ( aspell_error_number( possibleErr ) != 0 ) {
        MessageBox::msg( aspell_error_message( possibleErr ) );
        FOREACH( checker, checkers ) delete_aspell_speller( checker->second );
        return false;
    }
    else 
        checkers[name] = to_aspell_speller( possibleErr );

    if ( checkers.size() == 1 ) 
        FOREACH( chat, chat_manager->chats() ) ChatCreated( (*chat)->uins() );        

    return true;
}

void SpellChecker::RemoveCheckedLang( QString& name )
{
    VAR( checker, checkers.find( name ) );
    if ( checker != checkers.end() ) {
        delete_aspell_speller( checker->second );
        checkers.erase( name );
    }
}

bool SpellChecker::BuildCheckers()
{
    FOREACH( checker, checkers ) delete_aspell_speller( checker->second );
    checkers.clear();

    // load languages to check from configuration
    QString checkedStr = config->readEntry( "Languages", "Checked", "pl");
    QStringList checkedList = QStringList::split( ',', checkedStr );

    if ( config->readBoolEntry( "Ignore", "Accents", false ) )
        aspell_config_replace( spellConfig, "ignore-accents", "true" ); 
    else
        aspell_config_replace( spellConfig, "ignore-accents", "false" ); 
    if ( config->readBoolEntry( "Ignore", "Case", false ) )
        aspell_config_replace( spellConfig, "ignore-case", "true" ); 
    else
        aspell_config_replace( spellConfig, "ignore-case", "false" ); 

    // create aspell checkers for each language
    for (unsigned int i = 0; i < checkedList.count(); i++) 
        if ( !AddCheckedLang( checkedList[i] ) ) {
            delete_aspell_config( spellConfig );
            delete config;
            return false;
        };
    return true;
}

void SpellChecker::BuildMarkTag()
{
    VAR ( openChats, chat_manager->chats () );
    FOREACH ( chat, openChats ) CleanMessage( *chat );

    beginMark = "<span style=\"";

    if ( config->readBoolEntry( "Mark", "Bold", false ) ) 
        beginMark += "font-weight:600;";
    if ( config->readBoolEntry( "Mark", "Italic", false ) ) 
        beginMark += "font-style:italic;";
    if ( config->readBoolEntry( "Mark", "Underline", false ) ) 
        beginMark += "text-decoration:underline;";
    QColor colorMark("#FF0101");
    colorMark = config->readColorEntry( "Mark", "Color", &colorMark );
    beginMark += "color:" + colorMark.name() + "\">";
}

void SpellChecker::ChangeMarkColor( const QColor& col )
{
    QString normalized = col.name();
    normalized[2] = '1';
    normalized[4] = '2';
    normalized[6] = '3';
    config->writeEntry( "Mark", "Color", QColor( normalized ) );
}

void SpellChecker::SaveConfig()
{
    config->writeEntry( "Languages", "Checked", CheckedLanguages().join(",") );
    config->writeEntry( "Ignore", "Accents", 
                                                config->readBoolEntry( "Ignore", "Accents", false ) );
    config->writeEntry( "Ignore", "Case", 
                                                config->readBoolEntry( "Ignore", "Case", false ) );

    QColor colorMark("#FF0101");
    config->writeEntry( "Mark", "Color", 
                                                config->readColorEntry( "Mark", "Color", &colorMark ) );
    config->writeEntry( "Mark", "Bold", 
                                                config->readBoolEntry( "Mark", "Bold", false ) );
    config->writeEntry( "Mark", "Italic", 
                                                config->readBoolEntry( "Mark", "Italic", false ) );
    config->writeEntry( "Mark", "Underline", 
                                                config->readBoolEntry( "Mark", "Underline", false ) );

    config->sync();
}

void SpellChecker::ChatCreated( const UinsList& interlocutors )
{
    if ( checkers.size() > 0 ) {
        if ( !myWakeupTimer->isActive() ) 
            myWakeupTimer->start( 200 );
 
        connect( chat_manager->findChatByUins( interlocutors ), 
                         SIGNAL(messageSendRequested(Chat*)),
                         this, SLOT(CleanMessage(Chat*)) );
 //        SpellLighter* s = new SpellLighter( chat_manager->findChatByUins( interlocutors ) 
 //                                                                       ->edit );
    }
}

void SpellChecker::CleanMessage( Chat* chat )
{
    HtmlDocument parsedHtml;
    parsedHtml.parseHtml( chat->edit->text() );
    bool change = false;

    for( int i = 0; i < parsedHtml.countElements(); i++) 
        if ( IsTagMyOwn( parsedHtml, i ) ) {
            parsedHtml.setElementValue( i, "" );
            i++;
            parsedHtml.setElementValue( i + 1, "" );
            i++;
            change = true;
        };

    // if we have changed contents of chat window, than update it
    if ( change ) UpdateChat( chat->edit, parsedHtml.generateHtml() );
}

void SpellChecker::ExecuteChecking()
{
    if ( chat_manager->chats().size() == 0 || checkers.size() == 0 )
        myWakeupTimer->stop();

    // iterate through open chats and check their inputs
    VAR (openChats, chat_manager->chats ());
    FOREACH (chat, openChats) {
        HtmlDocument parsedHtml;
        parsedHtml.parseHtml( (*chat)->edit->text() );
        bool change = false;

        for( int i = 0; i < parsedHtml.countElements(); i++) 
        {
 /*
            if ( change ) {
                printf("dump html\n");
                for( int i = 0; i < parsedHtml.countElements(); i++) 
                    {
                        printf ("%s\n", (const char* const) parsedHtml.elementText( i ).utf8() );
                    }
                printf("/dump html\n");
            };
    */
            if( parsedHtml.isTagElement( i ) ) continue;

            QString text = parsedHtml.elementText( i );
            bool inWhite = true;
            int lastBegin = -1, lastEnd = -1;

            for(unsigned int j = 0; j < text.length(); j++)
            {
                if ( inWhite ) {
                    if ( text[j].isLetter() ) {
                        inWhite = false;
                        lastBegin = j;
                    }
                }
                else {
                    // if we are at the end of current word
                    if ( !text[j].isLetter() || j == text.length() - 1 ) 
                    {
                        inWhite = true;
                        if ( text[j].isLetter() && j == text.length() - 1) {
                            lastEnd = j + 1;
                            
                            if ( i + 1 < parsedHtml.countElements() && 
                                     IsTagMyOwn( parsedHtml, i + 1 ) ) {
                                parsedHtml.splitElement( i, lastBegin, lastEnd - lastBegin );
                                parsedHtml.setElementValue( i + 2, parsedHtml.elementText(i) + 
                                                                                        parsedHtml.elementText(i+2), false );
                                parsedHtml.setElementValue( i, "");
                                
                                continue;
                            }
                        }
                        else lastEnd = j;

                        QString word = text.mid( lastBegin, lastEnd - lastBegin );
                        QCString wordUtf8 = word.utf8();

                        // run checkers for all languages to check if this word is
                        // valud in some of them
                        bool isWordValid = checkers.size() == 0;
                        FOREACH( checker, checkers )
                            if ( aspell_speller_check( checker->second, wordUtf8, -1 ) ) {
                                isWordValid = true;
                                break;
                            };

                        if ( !isWordValid ) 
                        {
                            // current word isn't in dictionary, so take a look at it
                            parsedHtml.splitElement( i, lastBegin, lastEnd - lastBegin );

                            // check if this word isn't already marked as misspelled
                            if ( (i == 0 || !IsTagMyOwn( parsedHtml, i - 1 )) &&
                                    i < parsedHtml.countElements() - 1 &&
                                    !parsedHtml.isTagElement( i + 1 ) )
                            {
                                parsedHtml.insertTag( i, beginMark );
                                parsedHtml.insertTag( i + 2, endMark );
                                change = true;
                            }
                            else if ( i > 0 && i < parsedHtml.countElements() ) {
                                // word is currently marked, but we must check if we don't
                                // have some extra charactes inserted between this word 
                                // and the endmark
                                if ( !parsedHtml.isTagElement( i + 1 ) ) {
                                    parsedHtml.setElementValue( i + 2, parsedHtml.elementText( i + 1 ), 
                                                                                            false );
                                    parsedHtml.setElementValue( i + 1, endMark, true );
                                    change = true;
                                }
                            }
                            
                            break;
                        }
                        else {
                            // current word is correct, so remove possible tags
                            if ( i > 0 && IsTagMyOwn( parsedHtml, i - 1 ) &&
                                     i < parsedHtml.countElements() - 1 && 
                                     parsedHtml.isTagElement( i + 1 ) )
                            {    
                                // we bring word back to not marked
                                parsedHtml.setElementValue( i - 1, "");
                                parsedHtml.setElementValue( i + 1, "");

                                // trigger chat update
                                change = true;
                            }
                        }
                    }
                }
            }
        }
        // if we have changed contents of chat window, than update it
        if ( change ) UpdateChat( (*chat)->edit, parsedHtml.generateHtml() );

        //             printf ( "%s\n", (const char* const) (*chat)->edit->text().ascii() );
    }
}

void SpellChecker::UpdateChat( CustomInput* edit, QString text )
{
    int currentY, currentX;
    edit->getCursorPosition(&currentY, &currentX);
    edit->setUpdatesEnabled( false );
    //    HtmlDocument::unescapeText( text );
    edit->setText( text );
    // set cursor to initial position
    edit->setCursorPosition(currentY, currentX);
    edit->setUpdatesEnabled( true );
}

// check some part of element content to distiguish it from
// tags generated by external code
bool SpellChecker::IsTagMyOwn( HtmlDocument& doc, int idx)
{
    unsigned int len = beginMark.length();
    if ( doc.isTagElement ( idx ) ) {
        QString text = doc.elementText( idx );
        return text.length() == len && text[len - 3] == beginMark[len - 3] &&
            text[len - 5] == beginMark[len - 5] && 
            text[len - 7] == beginMark[len - 7];
    }
    else
        return false;
}

SpellChecker* spellcheck;

SpellLighter::SpellLighter (QTextEdit * textEdit) : QSyntaxHighlighter (textEdit) {
}

int SpellLighter::highlightParagraph (const QString &text, int endStateOfLastPar) 
{
    printf ("%s\n", (const char* const) text.utf8() );
    //    if (text.length () > 5 && text.length () < 8) setFormat ( 2, 2, QColor ("#FF0101") );
    return 0;
}
